class Color {
  static final Color black = Color(0, 0, 0);
  static final Color white = Color(255, 255, 255);

  static final Color gray = Color(128, 128, 128);
  static final Color lightGray = Color(64, 64, 64);
  static final Color darkGray = Color(192, 192, 192);

  static final Color red = Color(235, 0, 0);
  static final Color lightRed = Color(255, 90, 90);
  static final Color darkRed = Color(155, 15, 15);

  static final Color orange = Color(255, 160, 0);
  static final Color lightOrange = Color(255, 192, 76);
  static final Color darkOrange = Color(204, 132, 0);

  static final Color gold = Color(255, 215, 0);
  static final Color lightGold = Color(255, 227, 76);
  static final Color darkGold = Color(229, 193, 0);

  static final Color yellow = Color(255, 255, 0);
  static final Color lightYellow = Color(255, 255, 153);
  static final Color darkYellow = Color(229, 229, 0);

  static final Color green = Color(0, 128, 0);
  static final Color lightGreen = Color(76, 166, 76);
  static final Color darkGreen = Color(0, 89, 0);

  static final Color cyan = Color(0, 255, 255);
  static final Color lightCyan = Color(153, 255, 255);
  static final Color darkCyan = Color(0, 204, 204);

  static final Color blue = Color(0, 0, 255);
  static final Color lightBlue = Color(76, 76, 255);
  static final Color darkBlue = Color(0, 0, 110);

  static final Color purple = Color(128, 0, 128);
  static final Color lightPurple = Color(166, 76, 166);
  static final Color darkPurple = Color(112, 0, 112);

  static final Color pink = Color(255, 192, 203);
  static final Color lightPink = Color(255, 223, 229);
  static final Color darkPink = Color(229, 172, 182);

  static final Color brown = Color(97, 48, 13);
  static final Color lightBrown = Color(128, 89, 61);
  static final Color darkBrown = Color(67, 33, 9);

  final int r;
  final int g;
  final int b;

  String get cssString => "rgb($r, $g, $b)";

  int get hashCode => r.hashCode ^ g.hashCode ^ b.hashCode;

  @override
  bool operator ==(Object other) {
    if (other is Color) return other.r == r && other.g == g && other.b == b;

    return false;
  }

  Color(this.r, this.g, this.b);

  Color add(Color other, [double fraction]) {
    fraction ??= 1.0;

    int newR = (this.r + other.r * fraction).toInt().clamp(0, 255);
    int newG = (this.g + other.g * fraction).toInt().clamp(0, 255);
    int newB = (this.b + other.b * fraction).toInt().clamp(0, 255);

    return Color(newR, newG, newB);
  }

  Color blend(Color other, double fraction) {
    double fractionThis = 1.0 - fraction;

    int newR = (this.r * fractionThis + other.r * fraction).toInt();
    int newG = (this.g * fractionThis + other.g * fraction).toInt();
    int newB = (this.b * fractionThis + other.b * fraction).toInt();

    return Color(newR, newG, newB);
  }

  Color blendPercent(Color other, int percent) => blend(other, percent / 100);
}
